/*
 * Copyright (C)2003 USAGI/WIDE Project
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/*
 * Authors:
 *	Noriaki TAKAMIYA @USAGI
 *	Masahide NAKAMURA @USAGI
 *	YOSHIFUJI Hideaki @USAGI
 */
#ifndef _MIP6_H
#define _MIP6_H

#include <linux/skbuff.h>
#include <linux/in6.h>

extern int mip6_init(void);

/*
 * Mobile IPv6 Destination Header: Home Address Option
 */
extern int mip6_dest_hao(struct sk_buff *skb, int optoff);


/* XXX: Home Address Option in Destination Option Header */
struct destopt_hao
{
	__u8			type;
	__u8			length;
	struct in6_addr		addr;	/* Home Address */
} __attribute__ ((__packed__));


/*
 * Mobility Header
 */
struct mip6_mh_hdr {
	__u8	nexthdr;
	__u8	hdrlen;
	__u8	type;
	__u8	reserved;
	__u16	cksum;
	__u8	data[0];
} __attribute__ ((__packed__));

struct mip6_mh_brr {
	__u16	reserved;
	__u8	opt[0];
} __attribute__ ((__packed__));

struct mip6_mh_test_init {
	__u16	reserved;
	__u8	cookie[8];
	__u8	opt[0];
} __attribute__ ((__packed__));

struct mip6_mh_test {
	__u16	nonce_index;
	__u8	cookie[8];
	__u8	keygen[8];
	__u8	opt[0];
} __attribute__ ((__packed__));

struct mip6_mh_bu {
	__u16	seq;
	__u8	flags;
	__u8	reserved;
	__u16	lifetime;
	__u8	opt[0];
} __attribute__ ((__packed__));

struct mip6_mh_ba {
	__u8	status;
	__u8	flags;
	__u16	seq;
	__u16	lifetime;
	__u8	opt[0];
} __attribute__ ((__packed__));

struct mip6_mh_be {
	__u8	status;
	__u8	reserved;
	struct in6_addr hoa;
} __attribute__ ((__packed__));


/*
 * Mobility Header Type
 */
#define	MIP6_MH_BRR	0
#define	MIP6_MH_HOTI	1
#define	MIP6_MH_COTI	2
#define	MIP6_MH_HOT	3
#define	MIP6_MH_COT	4
#define	MIP6_MH_BU	5
#define	MIP6_MH_BA	6
#define	MIP6_MH_BE	7

struct mip6_mh_opt {
	__u8	type;
	__u8	len;
	__u8	data[0];
} __attribute__ ((__packed__));

struct mip6_mh_opt_pad_1 {
	__u8	type;
} __attribute__ ((__packed__));

struct mip6_mh_opt_pad_n {
	__u8	type;
	__u8	len;
	__u8	padding[0];	/* variable length */
} __attribute__ ((__packed__));

struct mip6_mh_opt_brr {
	__u8	type;
	__u8	len;
	__u16	refresh_interval;
} __attribute__ ((__packed__));

struct mip6_mh_opt_alt_coa {
	__u8	type;
	__u8	len;
	struct in6_addr	alt_coa;
} __attribute__ ((__packed__));

struct mip6_mh_opt_nonce_index {
	__u8	type;
	__u8	len;
	__u16	home_nonce_index;
	__u16	careof_nonce_index;
} __attribute__ ((__packed__));

struct mip6_mh_opt_binding_auth {
	__u8	type;
	__u8	len;
	__u8	authdata[12];	/* XXX: variable length? */
} __attribute__ ((__packed__));


/*
 * Mobility Option Type
 */
#define	MIP6_OPT_PAD_1		0
#define	MIP6_OPT_PAD_N		1
#define	MIP6_OPT_BRR		2
#define	MIP6_OPT_ALT_COA	3
#define	MIP6_OPT_NONCE_INDEX	4
#define	MIP6_OPT_BINDING_AUTH	5


/*
 * Status in BA
	   0 Binding Update accepted
           1 Accepted but prefix discovery necessary
         128 Reason unspecified
         129 Administratively prohibited
         130 Insufficient resources
         131 Home registration not supported
         132 Not home subnet
         133 Not home agent for this mobile node
         134 Duplicate Address Detection failed
         135 Sequence number out of window
         136 Expired home nonce index
         137 Expired care-of nonce index
         138 Expired nonces
*/
#define MIP6_BA_ACCEPTED			0
#define MIP6_BA_NEED_PREFIX_DISC		1
#define MIP6_BA_UNSPEC				128
#define MIP6_BA_ADMIN				129
#define MIP6_BA_INSUFFICIENT_RESOURCE		130
#define MIP6_BA_NOT_SUPPORTED			131
#define MIP6_BA_NOT_HOME_SUBNET			132
#define MIP6_BA_NOT_HOME_AGENT			133
#define MIP6_BA_DAD_FAILED			134
#define MIP6_BA_SEQ_OUT_OF_WINDOW		135
#define	MIP6_BA_EXPIRED_HOME_NONCE_INDEX	136
#define	MIP6_BA_EXPIRED_CAREOF_NONCE_INDEX	137
#define	MIP6_BA_EXPIRED_NONCE			138

/*
 * Status in BE
 */
#define MIP6_BE_UNKNOWN_BINDING	1
#define MIP6_BE_UNRECOGNIZED_MH	2

/*
 * Flags in BU
 */
#define	MIP6_BU_F_ACK	0x80
#define	MIP6_BU_F_HR	0x40
#define	MIP6_BU_F_LL	0x20
#define	MIP6_BU_F_KM	0x10

/*
 * Flags in BA
 */
#define MIP6_BA_F_KM	0x80


#define MIP6_BC_HASH_SIZE 32
#define MIP6_BUL_HASH_SIZE 32

/*
 * Binding Cache Entry
 */
struct mip6_bc_entry {
	struct mip6_bc_entry *right;
	struct mip6_bc_entry *left;

	struct in6_addr	hoa;
	struct in6_addr	coa;
	__u16	lifetime;
	__u8	flags;
	__u16	seq;

	/* If any, add usage information for this Binding Cache entry.
	 * This is needed to implement the cache replacement
	 * policy in use in the Binding Cache.
	 */
};

/*
 * Binding Cache
 */
struct mip6_bc {
	rwlock_t lock;
	struct mip6_bc_entry *entry_hash_table[MIP6_BC_HASH_SIZE];
};

/*
 * structure of keygen information included in Home/Care-of Test
 */
struct keygen_info {
	__u8			keygen[8];
	__u16			nonce_index;
	__u32			recv_time;
};

/*
 * structure of Binding Update List
 */
struct mip6_bul {
	rwlock_t lock;
	struct mip6_bul_entry	*entry_hash_table[MIP6_BUL_HASH_SIZE];
};

struct mip6_bul_entry {
	struct mip6_bul_entry	*right;
	struct mip6_bul_entry	*left;
	struct in6_addr		cn_addr;
	struct in6_addr		hoa;
	struct in6_addr		coa;
	__u16			init_lifetime;
	__u16			current_lifetime;
	__u16			seq;
	__u32			sent_time_bu;
	__u8			state;
	__u8			need_bu;
	/* for Return Routability */
	__u32			sent_time_hoti;
	__u32			sent_time_coti;
	__u8			rr_state;	/* XXX: same as state? */
	__u8			home_cookie[8];
	__u8			careof_cookie[8];
	struct keygen_info	home_keygen_info;
	struct keygen_info	careof_keygen_info;
};

#endif
